# -*- coding: utf-8 -*-

"""
MEDIA:
    La Prensa, Honduras

USAGE:
    $ cd prensaHn/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##

    scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=7
"""

import scrapy, re, json
from datetime import datetime, date
from prensaHn.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class ImportantFlowData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    news_section = scrapy.Field()
    section_url  = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        base_url = "http://www.laprensa.hn/"
        
        section_list = ["economia", "mundo", "tecnologia", "cine", "cultura", "turismo",
                        "honduras", "sucesos", "espectaculos", "deportes"]

        if year is not None and month is not None and day is not None:
            self.stop_date = date(int(year), int(month), int(day))
            
            for s in section_list:
                flow_info = ImportantFlowData()
                flow_info['to_next_page'] = False
                flow_info['news_section'] = s
                flow_info['section_url']  = base_url + s + "/"
                
                request = scrapy.Request(url=base_url + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                yield request

        elif year is None and month is None and day is None:
            for s in section_list:
                flow_info = ImportantFlowData()
                flow_info['news_section'] = s
                flow_info['section_url']  = base_url + s + "/"

                request = scrapy.Request(url=base_url + s, callback=self.parse)
                request.meta['item'] = flow_info
                yield request

        else:
            print "Unable to execute this crawler with current given parameters."
            print "Enter all parameters: year, month and day, or none of them."



    def parse(self, response):
        flow_info    = response.meta['item']
        news_section = flow_info['news_section']
        section_url  = flow_info['section_url']
        link_list = []

        if news_section == "deportes":
            section = news_section.capitalize()
        elif news_section == "espectaculos":
            section = u'Expect\xc3\xa1culos'
        else:
            section = response.xpath('//div[@id="section_title"]/h1/a').extract_first()
            if section is not None : section = remove_tags(section)

        for entry in response.css('article.grid').css('div.content'):
            if section in entry.css('a.category').extract_first():
                link_list.append(entry.xpath('./a[2]/@href').extract_first())

        for link in link_list:
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.xpath('//ul[@id="paginacion"]').css('span.next > a::attr(href)').extract_first()
        if next_page is not None:
            flow_info = ImportantFlowData()
            flow_info['news_section'] = news_section
            flow_info['section_url']  = section_url
            
            request = scrapy.Request(url=section_url + next_page, callback=self.parse)
            request.meta['item'] = flow_info
            yield request



    def parse_with_stop_date(self, response):
        flow_info    = response.meta['item']
        news_section = flow_info['news_section']
        section_url  = flow_info['section_url']

        if not flow_info['to_next_page']:
            link_list = []

            if news_section == "deportes":
                section = news_section.capitalize()
            elif news_section == "espectaculos":
                section = u'Expect\xc3\xa1culos'
            else:
                section = response.xpath('//div[@id="section_title"]/h1/a').extract_first()
                if section is not None : section = remove_tags(section)

            for entry in response.css('article.grid').css('div.content'):
                if section in entry.css('a.category').extract_first():
                    link_list.append(entry.xpath('./a[2]/@href').extract_first())

            for link in link_list:
                flow_info = ImportantFlowData()
                flow_info['news_section'] = news_section
                flow_info['section_url']  = section_url
                flow_info['return_url']   = response.url

                if link_list.index(link) == link_list.index(link_list[-1]):
                    flow_info['is_last_link'] = True
                else:
                    flow_info['is_last_link'] = False

                request = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                yield request

        else:
            next_page = response.xpath('//ul[@id="paginacion"]').css('span.next > a::attr(href)').extract_first()
            
            if next_page is not None:
                flow_info = ImportantFlowData()
                flow_info['to_next_page'] = False
                flow_info['news_section'] = news_section
                flow_info['section_url']  = section_url
                
                request = scrapy.Request(url=section_url + next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                yield request



    def parse_item(self, response):
        item = NoticiasItem()
        resp = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        news_data = json.loads(resp)

        try:
            topic = news_data['articleSection']
        except:
            topic = None

        ## News item info ##
        item['date']  = news_data['datePublished'][:-1]
        item['title'] = news_data['headline']
        item['topic'] = topic
        item['text']  = news_data['articleBody']
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        resp = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
        news_data = json.loads(resp)
        news_date = news_data['datePublished']
        news_date = news_date[:news_date.find("T")]
        news_date = datetime.strptime(news_date, '%Y-%m-%d').date()

        if news_date >= self.stop_date:
            flow_info = response.meta['item']
            item = NoticiasItem()

            try:
                topic = news_data['articleSection']
            except:
                topic = None

            ## News item info ##
            item['date']  = news_data['datePublished'][:-1]
            item['title'] = news_data['headline']
            item['topic'] = topic
            item['text']  = news_data['articleBody']
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                yield request
