# -*- coding: utf-8 -*-

"""
MEDIA:
    La Jornada Baja California, Baja California

USAGE:
    $ cd laJornadaBC/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    
    $ scrapy crawl noticias --nolog -s filename=2017-02-21.json -a year=2017 -a month=2 -a day=21
"""

import scrapy, re, json
from datetime import datetime, date
from laJornadaBC.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class ImportantFlowData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    return_url   = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        self.base_url = "http://jornadabc.mx"
        section_list = ["baja-california", "chihuahua", "mexico", "mundo", "cultura", "espectaculos", "deportes"]

        if year is not None and month is not None and day is not None:
            self.stop_date = date(int(year), int(month), int(day))
            
            for s in section_list:
                flow_info = ImportantFlowData()
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=self.base_url + "/seccion/" + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                yield request

        elif year is None and month is None and day is None:
            for s in section_list:
                yield scrapy.Request(url=self.base_url + "/seccion/" + s, callback=self.parse)

        else:
            print "Unable to execute this crawler with current given parameters."
            print "Enter all parameters: year, month and day, or none of them."



    def parse(self, response):
        link_list = response.css('div.region-hard-first').css('div.views-field-title > span > a::attr(href)').extract()

        for link in link_list:
            yield scrapy.Request(url=self.base_url + link, callback=self.parse_item)

        next_page = response.css('li.pager-next > a::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=self.base_url + next_page, callback=self.parse)



    def parse_with_stop_date(self, response):
        flow_info = response.meta['item']

        if not flow_info['to_next_page']:
            link_list = response.css('div.region-hard-first').css('div.views-field-title > span > a::attr(href)').extract()

            for link in link_list:
                flow_info = ImportantFlowData()
                flow_info['return_url'] = response.url

                if link_list.index(link) == link_list.index(link_list[-1]):
                    flow_info['is_last_link'] = True
                else:
                    flow_info['is_last_link'] = False

                request = scrapy.Request(url=self.base_url + link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                yield request

        else:
            next_page = response.css('li.pager-next > a::attr(href)').extract_first()

            if next_page is not None:
                flow_info = ImportantFlowData()
                flow_info['to_next_page'] = False
                
                request = scrapy.Request(url=self.base_url + next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                yield request



    def parse_item(self, response):
        item = NoticiasItem()

        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

        title = response.css('div.block-content > h1 > a').extract_first()
        if title is not None : title = remove_tags(title)

        topic = response.css('span.section').extract_first()
        if topic is not None : topic = remove_tags(topic)

        text = response.css('div.region-article-second').css('div.block-content').css('div.rtejustify').extract_first()
        if text is not None:
            text = remove_tags(text)

        elif text is None or text == '':
            text = ''
            for p in response.css('div.region-article-second').css('div.block-content').css('div.field-items').css('p').extract():
                text += remove_tags(p) + "\n"

        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        news_date = news_date[:news_date.find("T")]
        news_date = datetime.strptime(news_date, '%Y-%m-%d').date()

        if news_date >= self.stop_date:
            flow_info = response.meta['item']
            item = NoticiasItem()

            news_date = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()

            title = response.css('div.block-content > h1 > a').extract_first()
            if title is not None : title = remove_tags(title)

            topic = response.css('span.section').extract_first()
            if topic is not None : topic = remove_tags(topic)

            text = response.css('div.region-article-second').css('div.block-content').css('div.rtejustify').extract_first()
            if text is not None:
                text = remove_tags(text)

            elif text is None or text == '':
                text = ''
                for p in response.css('div.region-article-second').css('div.block-content').css('div.field-items').css('p').extract():
                    text += remove_tags(p) + "\n"

            ## News item info ##
            item['date']  = news_date
            item['title'] = title
            item['topic'] = topic
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                yield request
