# -*- coding: utf-8 -*-

"""
MEDIA:
    La Jornada Maya, Yucatán

USAGE:
    $ cd laJornadaMaya/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date. ##
    
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    
    $ scrapy crawl noticias --nolog -s filename=2018-08-10.json -a year=2018 -a month=8 -a day=10
"""

import scrapy, re, json
from datetime import datetime, date
from laJornadaMaya.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class ImportantFlowData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    to_next_page = scrapy.Field()
    is_last_link = scrapy.Field()
    return_url   = scrapy.Field()
    current_page = scrapy.Field()
    news_section = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        self.base_url = "https://www.lajornadamaya.mx/"
        section_list = ["yucatan", "quintana-roo", "campeche", "nacional", "internacional", "deportes", "opinion"]

        if year is not None and month is not None and day is not None:
            self.stop_date = date(int(year), int(month), int(day))
            
            for s in section_list:
                flow_info = ImportantFlowData()
                flow_info['to_next_page'] = False
                flow_info['news_section'] = s
                
                request = scrapy.Request(url=self.base_url + s, callback=self.parse_with_stop_date_1)
                request.meta['item'] = flow_info
                yield request

        elif year is None and month is None and day is None:
            self.stop_date = None
            
            for s in section_list:
                flow_info = ImportantFlowData()
                flow_info['news_section'] = s

                request = scrapy.Request(url=self.base_url + s, callback=self.parse)
                request.meta['item'] = flow_info
                yield request

        else:
            print "Unable to execute this crawler with current given parameters."
            print "Enter all parameters: year, month and day, or none of them."



    def parse(self, response):
        flow_info = response.meta['item']
        s = flow_info['news_section']

        link_list = response.css('div.slide').css('h1.title > a::attr(href)').extract()
        link_list.extend(response.css('div.moreArticles').css('h2.title > a::attr(href)').extract())

        for link in link_list:
            item_info = ImportantFlowData()
            item_info['news_section'] = s
            
            reqst = scrapy.Request(url=self.base_url + link, callback=self.parse_item)
            reqst.meta['item'] = item_info
            yield reqst

        p = 0
        flow_info = ImportantFlowData()
        flow_info['current_page'] = p
        flow_info['news_section'] = s

        request = scrapy.Request(url=self.base_url + s + "?p=" + str(p), callback=self.parse_2)
        request.meta['item'] = flow_info
        yield request



    def parse_2(self, response):
        flow_info = response.meta['item']
        p = flow_info['current_page']
        s = flow_info['news_section']

        news_list = json.loads(response.body)
        if len(news_list) > 0:
            url = self.base_url[:self.base_url.rfind("/")]
            
            for news in news_list:
                item_info = ImportantFlowData()
                item_info['news_section'] = s

                reqst = scrapy.Request(url=url + news['url'], callback=self.parse_item)
                reqst.meta['item'] = item_info
                yield reqst

        p += 1
        flow_info['current_page'] = p

        request = scrapy.Request(url=self.base_url + s + "?p=" + str(p), callback=self.parse_2)
        request.meta['item'] = flow_info
        yield request



    def parse_with_stop_date_1(self, response):
        flow_info = response.meta['item']
        s = flow_info['news_section']

        if not flow_info['to_next_page']:
            link_list = response.css('div.slide').css('h1.title > a::attr(href)').extract()
            link_list.extend(response.css('div.moreArticles').css('h2.title > a::attr(href)').extract())

            for link in link_list:
                item_info = ImportantFlowData()
                item_info['news_section'] = s
                item_info['return_url']   = response.url

                if link_list.index(link) == link_list.index(link_list[-1]):
                    item_info['is_last_link'] = True
                else:
                    item_info['is_last_link'] = False

                request = scrapy.Request(url=self.base_url + link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = item_info
                yield request

        else:
            flow_info = ImportantFlowData()
            p = 0
            flow_info['current_page'] = p
            flow_info['news_section'] = s
            
            request = scrapy.Request(url=self.base_url + s + "?p=" + str(p), callback=self.parse_with_stop_date_2)
            request.meta['item'] = flow_info
            yield request



    def parse_with_stop_date_2(self, response):
        flow_info = response.meta['item']
        p = flow_info['current_page']
        s = flow_info['news_section']

        news_list = json.loads(response.body)
        if len(news_list) > 0:
            url = self.base_url[:self.base_url.rfind("/")]
            
            for news in news_list:
                news_date = datetime.strptime(news['publishDate'], '%Y-%m-%d %H:%M:%S').date()

                if news_date >= self.stop_date:
                    item_info = ImportantFlowData()
                    item_info['news_section'] = s

                    reqst = scrapy.Request(url=url + news['url'], callback=self.parse_item)
                    reqst.meta['item'] = item_info
                    yield reqst

                    if news_list.index(news) == news_list.index(news_list[-1]):
                        p += 1
                        flow_info = ImportantFlowData()
                        flow_info['current_page'] = p
                        flow_info['news_section'] = s

                        request = scrapy.Request(url=self.base_url + s + "?p=" + str(p), callback=self.parse_with_stop_date_2)
                        request.meta['item'] = flow_info
                        yield request

                else:
                    break



    def parse_item(self, response):
        flow_info = response.meta['item']
        item = NoticiasItem()
        text = ''

        news_date = response.url[:response.url.rfind("/")]
        news_date = news_date[news_date.rfind("/")+1:]
        news_date = datetime.strptime(news_date, '%Y-%m-%d').isoformat("T")

        title = response.xpath('//article/h1').extract_first()
        if title is not None : title = remove_tags(title)

        topic = flow_info['news_section'].capitalize()

        for p in response.xpath('//article').css('div.txt').extract():
                p = p.replace("<br>", "\n")
                text += remove_tags(p) + "\n"

        ## News item info ##
        item['date']  = news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        news_date = response.url[:response.url.rfind("/")]
        news_date = news_date[news_date.rfind("/")+1:]
        news_date = datetime.strptime(news_date, '%Y-%m-%d').date()

        if news_date >= self.stop_date:
            flow_info = response.meta['item']
            item = NoticiasItem()
            text = ''

            news_date = datetime.strptime(news_date.isoformat(), '%Y-%m-%d').isoformat("T")

            title = response.xpath('//article/h1').extract_first()
            if title is not None : title = remove_tags(title)

            topic = flow_info['news_section'].capitalize()

            for p in response.xpath('//article').css('div.txt').extract():
                p = p.replace("<br>", "\n")
                text += remove_tags(p) + "\n"

            ## News item info ##
            item['date']  = news_date
            item['title'] = title
            item['topic'] = topic
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['is_last_link']:
                flow_info['to_next_page'] = True
                
                request = scrapy.Request(url=flow_info['return_url'], callback=self.parse_with_stop_date_1, dont_filter=True)
                request.meta['item'] = flow_info
                
                yield request
