# -*- coding: utf-8 -*-

"""
MEDIA:
    La Tribuna, Honduras

USAGE:
    ## Get all the news from a specific date. ##
    ---------------------------------------------------------------------------------------------
    $ cd tribunaHn/
    $ scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=2 -a day=29
"""

import scrapy, re
from tribunaHn.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """
    Class for Time Zone
    """
    def utcoffset(self, dt):
        ## Time zone for Honduras: UTC-6 ##
        return timedelta(hours=-6)

    def tzname(self, dt):
        ## Time zone name ##
        return 'UTC-6'



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"

    def start_requests(self):
        tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)
        day = getattr(self, 'day', None)

        self.news_date = datetime(int(year), int(month), int(day), tzinfo=tz).isoformat('T')
        
        baseURL = 'http://www.latribuna.hn/' + year + '/' + month + '/' + day

        yield scrapy.Request(url=baseURL, callback=self.parse)



    def parse(self, response):
        for link in response.xpath('//div[@id="main"]').css('h3 > a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)

        next_page = response.css('span.next > a::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)



    def parse_item(self, response):
        text = ''
        item = NoticiasItem()

        title = response.css('article.article-post').css('h1').extract_first()
        if title is not None: title = remove_tags(title)

        topic = None
        topic_list = response.css('aside.tags').css('li > a').extract()
        if len(topic_list) > 0:
            topic = remove_tags(topic_list[0])

        for p in response.css('div.article-post-content').css('p').extract():
            text += remove_tags(p) + '\n'


        ## News item info ##
        item['date']  = self.news_date
        item['title'] = title
        item['topic'] = topic
        item['text']  = text.strip()
        item['url']   = response.url
        
        yield item

